@@ -29,6 +29,8 @@
 #ifndef thparse_h
 #define thparse_h
 
+#include <stdio.h>
+#include <string>
 #include "thbuffer.h"
 #include "thmbuffer.h"
 
@@ -44,7 +46,7 @@ enum {
  */
 
 typedef struct {
-  char * s;  ///< String.
+  const char * s;  ///< String.
   int tok;  ///< Token.
 } thstok;
 
@@ -53,7 +55,7 @@ typedef struct {
  * Match string token.
  *
  * Scan array of token definitions (stok) for given string. If not found,
- * last element of array is returned. This array should allways be 
+ * last element of array is returned. This array should always be
  * alphabetically sorted and last element should be set to
  * like TT_UNKNOWN.
  *
@@ -64,15 +66,26 @@ typedef struct {
  * @sa thmatch_token
  */
  
-int thmatch_stok(char *buffer, const thstok *tab, int tab_size);
+int thmatch_stok(const std::string& token, const thstok *tab, std::size_t tab_size);
+ 
+int thcasematch_stok(const std::string& token, const thstok *tab, std::size_t tab_size);
 
 
 /**
- * Match string token macro.
+ * Helper functions to determine array size.
  */
 
-#define thmatch_token(buffer, table) thmatch_stok(buffer, table,\
-  ((sizeof(table)/sizeof(thstok)) - 1))
+template <std::size_t N>
+inline int thmatch_token(const std::string& token, const thstok (&table)[N])
+{
+  return thmatch_stok(token, table, N);
+}
+
+template <std::size_t N>
+inline int thcasematch_token(const std::string& token, const thstok (&table)[N])
+{
+  return thcasematch_stok(token, table, N);
+}
 
 
 /**
@@ -81,7 +94,7 @@ int thmatch_stok(char *buffer, const thstok *tab, int tab_size);
  * Inverse function to thmatch_stok().
  */
  
-char * thmatch_tstring(int token, const thstok *tab, int tab_size);
+const char * thmatch_tstring(int token, const thstok *tab, int tab_size);
 
 
 /**
@@ -100,7 +113,7 @@ enum {
   TT_UNKNOWN_BOOL = 0,  ///< Not match.
   TT_TRUE,  ///< yes, on, 1, true.
   TT_FALSE,  ///< no, off, 0, false.
-  TT_AUTO,  ///< automatical setting
+  TT_AUTO,  ///< automatic setting
 };
 
 
@@ -195,13 +208,19 @@ static const thstok thtt_special_val[] = {
  * Parse double value (with check for special values).
  */
  
-void thparse_double(int & sv, double & dv, char * src); 
+void thparse_double(int & sv, double & dv, const char * src); 
+
+/**
+ * Parse length value.
+ */
+ 
+void thparse_length(int & sv, double & dv, const char * src); 
 
 /**
  * Parse double value given in DMS (with check for special values).
  */
  
-void thparse_double_dms(int & sv, double & dv, char * src); 
+void thparse_double_dms(int & sv, double & dv, const char * src); 
 
 /**
  * Update double value (with check for special values).
@@ -214,14 +233,14 @@ void thparse_double_dms(int & sv, double & dv, char * src);
  * Split given string into first word and the rest.
  */
  
-void thsplit_word(thbuffer * dword, thbuffer * drest, char * src);
+void thsplit_word(thbuffer * dword, thbuffer * drest, const char * src);
 
 
 /**
  * Split given string into words (separated by white spaces).
  */
  
-void thsplit_words(thmbuffer * dest, char * src);
+void thsplit_words(thmbuffer * dest, const char * src);
 
 
 /**
@@ -239,7 +258,7 @@ void thsplit_strings(thmbuffer * dest, const char * src, const char separator);
  * Take care of C:\....
  */
  
-void thsplit_paths(thmbuffer * dest, char * src, char separator);
+void thsplit_paths(thmbuffer * dest, const char * src, char separator);
 
 
 /**
@@ -249,14 +268,14 @@ void thsplit_paths(thmbuffer * dest, char * src, char separator);
  * ("" = ") or given in [] brackets.
  */
 
-void thsplit_args(thmbuffer * dest, char * src);
+void thsplit_args(thmbuffer * dest, const char * src);
 
 
 /**
  * Separate path part from full file name.
  */
  
-void thsplit_fpath(thbuffer * dest, char * src);
+void thsplit_fpath(thbuffer * dest, const char * src);
 
 
 /**
@@ -265,14 +284,14 @@ void thsplit_fpath(thbuffer * dest, char * src);
  * Valid keyword characters are A-Z, a-z, 0-9,'_' and '-'.
  */
  
-bool th_is_keyword(char * str);
+bool th_is_keyword(const char * str);
 
 
 /**
  * Check if given string is index (positive integer).
  */
  
-bool th_is_index(char * str);
+bool th_is_index(const char * str);
 
 /**
  * Check if given string is keyword list.
@@ -283,7 +302,7 @@ bool th_is_index(char * str);
  * @param sep Keyword separator.
  */
  
-bool th_is_keyword_list(char * str, char sep);
+bool th_is_keyword_list(const char * str, char sep);
 
 
 /**
@@ -292,7 +311,7 @@ bool th_is_keyword_list(char * str, char sep);
  * A-Z, a-z, 0-9 and [_.,-/+*']
  */
  
-bool th_is_extkeyword(char * str);
+bool th_is_extkeyword(const char * str);
 
 
 /**
@@ -311,6 +330,7 @@ void thdecode_tcl(thbuffer * dest, const char * src);
 void thdecode_tex(thbuffer * dest, const char * src);
 void thdecode_utf2tex(thbuffer * dest, const char * src);
 void thdecode_sql(thbuffer * dest, const char * src);
+void thdecode_mp(thbuffer * dest, const char * src);
 
 
 /**
@@ -324,21 +344,21 @@ void thdecode_arg(thbuffer * dest, const char * src);
  * Parse altitude value
  */
  
-void thparse_altitude(char * src, double & altv, double & fixv);
+void thparse_altitude(const char * src, double & altv, double & fixv);
 
 
 /**
  * Check image.
  */
 
-void thparse_image(char * fname, double & width, double & height, double & dpi, int & type);
+void thparse_image(const char * fname, double & width, double & height, double & dpi, int & type);
 
 
 /**
- * Set color according to color map (TODO).
+ * Set color according to color map.
  */
  
-void thset_color(int color_map, double index, double total, double & R, double & G, double & B);
+void thset_color(int color_map, double index, double total, class thlayout_color & clr);
 
 
 /**
@@ -349,6 +369,35 @@ void thset_grid(double gorigin, double gsize, double min, double max, double & q
 
 
 
+/**
+ * Convert therion string to xhtml.
+ */
+ 
+const char * thutf82xhtml(const char * src);
+
+/**
+ * Base 64 file encoder.
+ */
+
+void thbase64_encode(const char * fname, FILE * fout);
+
+/**
+ * Check if path is absolute.
+ */
+
+bool thpath_is_absolute(const char * fname);
+
+/**
+ * Check on attribute name.
+ */
+bool th_is_attr_name(const char * str);
+
+std::string ths2tex(std::string original, int lang = -1, bool remove_kerning = false);
+
+std::string ths2txt(const char * original, int lang = -1, int encoding = -1);
+
+std::string ths2txt(std::string original, int lang = -1, int encoding = -1);
+
 #endif
 
 
